﻿//     Copyright (c) Sitecore Corporation 1999-2016
/* jshint unused: vars */
require.config({
    paths: {
        CommerceBasePage: "/sitecore/shell/client/Applications/MerchandisingManager/CommerceBasePage"
    }
});

define(["sitecore", "CommerceBasePage"], function (Sitecore, cbp) {
    var VariantDetail = cbp.extend({
        hasInventory: false,
        inventoryChanged: false,
        currentEditedMediaField: null,
        ajaxToken: {},
        tabStrip: null,
        initialized: function () {
            $("#ui-datepicker-div").hide();
            this.HelpProvider.set("isReady", true);
            this.HelpProvider.refresh();
            this.set("selectedTab", "{917CC948-91C6-40B0-9813-FF3AA89C1DA2}");

            cbp.prototype.initialized.call(this);

            this.listenTo(_sc, 'sc-concurrency-override', this.overridePreviousChanges);
            this.listenTo(_sc, 'sc-concurrency-cancel', this.cancelChanges);

            var token = Sitecore.Helpers.antiForgery.getAntiForgeryToken();
            this.ajaxToken[token.headerKey] = token.value;
            var variantId = CommerceUtilities.loadPageVar("target");
            this.set("targetId", variantId);
            this.VariantTab.on("change:selectedTab", this.selectedTabChanged, this);
            var self = this;

            // Select Media Dialog
            this.listenTo(_sc, 'sc-frame-message', this.addMedia);

            var language = CommerceUtilities.loadPageVar("lang");

            if (!language) {
                language = cbp.prototype.getCookie("commerceLang");
            }

            if (language) {
                this.set("currentLanguage", language);
            } else {
                this.set("currentLanguage", "en-US");
            }

            // Cache for faster selector lookup
            var variantTabElement = this.VariantTab.viewModel.$el;
            this.tabStrip = $("ul.sc-tabcontrol-navigation", variantTabElement).first();

            if (variantId) {
                // Set up tabs
                this.addLabelsToTabs();

                //For Concurrency Saving
                this.set("lastModified", null);
                this.set("overrideChanges", false);

                // Edit
                this.LanguageDataSource.set("commerceItemId", variantId);
                this.LanguageDataSource.set("isReady", true);
                this.LanguageDataSource.refresh();
                this.Language.on("change:items", this.setLanguage, this);

                this.VariantActionControl.set("isVisible", true);
                this.InventoryActionControl.set("isVisible", false);

                $.ajax({
                    url: "/api/sitecore/CommerceSearch/GetItem",
                    type: "POST",
                    context: this,
                    headers: this.ajaxToken,
                    data: {
                        id: variantId,
                        language: this.get("currentLanguage")
                    },
                    error: CommerceUtilities.IsAuthenticated,
                    success: function (data) {
                        if (data.InMaterializedCatalog) {
                            $(".cs-commerceitem :input").prop("disabled", true);
                            this.VariantActionControl.disableActionsNotInGroup("EnableWhenReadOnly");
                            this.InventoryActionControl.disableActionsNotInGroup("EnableWhenReadOnly");

                            $(".sc-ProductInventoryBaseFields :input").prop("disabled", true);
                            $(".sc-expander :input").prop("disabled", true);
                        }
                        self.HeaderTitle.set("text", data.DisplayName);
                        self.TitleControl.set("text", data.DisplayName);
                        self.CommerceImages.set("displayName", data.DisplayName);
                        self.CommerceImages.set("images", data.Variant_Images);

                        self.HeaderTitleLabel.set("text", self.ItemTypeDictionary.get("variant"));
                        self.set("itemPath", data.Path);

                        //For Concurrency Saving
                        self.set("lastModified", data.LastModified);
                    }
                });

                // Select Media from BaseUrl
                this.BaseUrlTextBox.viewModel.$el.keyup(this.updateBaseUrlPreview);
                this.BaseUrlTextBox.viewModel.$el.focusout(this.updateBaseUrlPreview);

                // Determine if we should display the inventory tab
                $.ajax({
                    url: "/api/sitecore/CommerceMetadata/DoesVariantHaveInventory",
                    type: "POST",
                    headers: this.ajaxToken,
                    data: {
                        id: variantId
                    },
                    error: CommerceUtilities.IsAuthenticated,
                    success: function (data) {
                        // If no associated inventory hide the tab.
                        var inventoryTab = $("li[data-tab-id='{CD3FF5FA-EA98-4965-91EE-2B7A400D0789}']", this.tabStrip);

                        if (data == "false") {
                            inventoryTab.hide();
                            self.hasInventory = false;
                            self.disableInventoryValidation();
                        } else if (data == "true") {
                            inventoryTab.show();
                            self.hasInventory = true;
                            self.loadInventoryDetails(variantId);
                            // Detect if inventory fields are dirty
                            $(".inventory").change(function () {
                                self.inventoryChanged = true;
                            });
                        }
                    }
                });
            } else {
                // Create
                this.VariantTab.set("selectedTab", "{917CC948-91C6-40B0-9813-FF3AA89C1DA2}");
                self.HeaderTitle.set("text", self.ItemTypeDictionary.get("Create Variant"));
                self.TitleControl.set("text", self.ItemTypeDictionary.get("Create Variant"));

                this.set("templateId", CommerceUtilities.loadPageVar("template"));
                this.set("parentId", CommerceUtilities.loadPageVar("parent"));
                this.VariantActionControl.set("isVisible", true);
                this.InventoryActionControl.set("isVisible", false);

                $("[data-sc-id='ProductInventoryBaseFields']").html("");
                $("[data-sc-id='InventorySkuFormRenderer']").html("");
                this.disableInventoryValidation();

                this.Language.set("isVisible", false);

                $("li[data-tab-id=\\{CD3FF5FA-EA98-4965-91EE-2B7A400D0789\\}]", this.tabStrip).hide();
            }

            // Setup event handler for the select media event of media field 
            var selectMediaEventHandler = $.proxy(this.showMediaDialog, this);
            this.on("selectMediaEvent", selectMediaEventHandler, this);

            // Setup event handler for select media accept changes
            this.listenTo(_sc, 'sc-frame-message', this.mediaItemSelected);
        },

        disableIfAX: function () {
            $(".cs-commerceitem :input").prop("disabled", true);
            this.VariantActionControl.disableActionsNotInGroup("EnableWhenReadOnly");
            this.InventoryActionControl.disableActionsNotInGroup("EnableWhenReadOnly");
            $(".sc-CommerceDateTime :input").prop("disabled", true);
        },

        disableInventoryValidation: function () {
            $('.inventory').removeClass('isrequired');
        },

        showMediaDialog: function (mediaField) {
            this.currentEditedMediaField = mediaField;
            this.SelectMediaDialogWindow.show();
        },

        mediaItemSelected: function (selectedMediaItemId) {

            if (this.currentEditedMediaField) {

                var mediaField = this.currentEditedMediaField;
                mediaField.model.set("mediaItemId", selectedMediaItemId);
            }

            this.setPageIsDirty(true);
            this.SelectMediaDialogWindow.hide();
            this.currentEditedMediaField = null;
        },

        setLanguage: function () {
            this.determineLanguageToSet();
            var l = this.get("currentLanguage");
            this.Language.set("selectedValue", l);
            this.Language.on("change:selectedItemId", this.selectedLanguageChange, this);
        },

        selectedLanguageChange: function () {
            var currentPath = this.updateQueryStringParameter(window.location.href, "lang", this.Language.get("selectedValue"));
            try {
                window.location.href = currentPath;
            } catch (e) {
                // stay on page
            }
        },

        addLabelsToTabs: function () {
            $("li[data-tab-id=\\{917CC948-91C6-40B0-9813-FF3AA89C1DA2\\}] > a", this.tabStrip).append("<label style=\"display:inline\"></label>");
            $("li[data-tab-id=\\{CD3FF5FA-EA98-4965-91EE-2B7A400D0789\\}] > a", this.tabStrip).append("<label style=\"display:inline\"></label>");
        },

        saveVariant: function () {
            var id = this.get("targetId");
            if (id) {
                this.updateVariant();
            } else {
                this.createVariant();
            }
        },

        updateVariant: function () {
            this.hideHeaderActionsMessage();
            var invalidFieldNames = [];
            if (!this.validateRequiredFields(invalidFieldNames)) {
                this.updateSaveButton(false);
                this.updateTabsIndicators();
                this.showRequiredFieldsMessages(invalidFieldNames);
                this.InProgress.hide();
                return;
            }

            this.displayInProgressMessage();
            var baseFields = this.VariantBaseFields.get("editFunctionBody");
            var currentLanguage = this.get("currentLanguage");

            var dataObject = {};
            this.addRendererFieldsToObject(dataObject, baseFields);

            var extendedFields = this.ExtendedFieldsFormRenderer.get("editFunctionBody");
            this.addRendererFieldsToObject(dataObject, extendedFields);

            dataObject.variant_Images = this.CommerceImages.get("images");
            dataObject.currentLanguage = currentLanguage;
            dataObject.itemId = this.get("targetId");

            //For Concurrency Saving
            dataObject.lastModified = this.get("lastModified");
            dataObject.overrideChanges = this.get("overrideChanges");

            if (this.hasInventory && this.inventoryChanged) {
                dataObject.inventory = {};
                this.addInventoryBaseFieldsToPost(dataObject.inventory);
                extendedFields = this.InventorySkuFormRenderer.get("editFunctionBody");
                this.addRendererFieldsToObject(dataObject.inventory, extendedFields);
            }

            $.ajax({
                url: "/api/sitecore/CommerceVariant/Update",
                type: "POST",
                headers: this.ajaxToken,
                data: dataObject,
                context: this,
                error: CommerceUtilities.IsAuthenticated,
                success: function (data) {
                    if (data.Status == "failed") {
                        this.hideInProgressMessage();
                        var self = this;
                        //this.updateSaveButton(false);

                        //For Concurrency Saving
                        this.set("overrideChanges", false);

                        for (i = 0; i < data.Errors.length; i++) {
                            this.HeaderActionsMessageBar.addMessage("error", data.Errors[i].ErrorMessage);
                            $("[data-sc-id='" + data.Errors[i].ControlId + "']").addClass("sc-validation");
                            this.serverUpdateTabErrorIndicator(data.Errors[i].ControlId);
                        }
                        this.HeaderActionsMessageBar.set("isVisible", true);
                        $(".sc-validation").change(function () {
                            self.updateValidationError($(this));
                        });
                        $(".sc-validation.sc-datepicker").click(function () {
                            self.updateValidationError($(this));
                        });
                    } else if (data.Status == "priorUpdate") {
                        this.hideInProgressMessage();
                        //For Concurrency Saving
                        this.ConcurrencyAlert.show();
                    } else {
                        this.hideInProgressMessage(true);
                        this.setPageIsDirty(false);
                        //For Concurrency Saving
                        this.set("overrideChanges", false);
                        if (data.Status.indexOf("success") === 0) {
                            this.set("lastModified", data.Status.split('|')[1]);
                        }
                    }
                }
            });
        },

        //For Concurrency Saving
        cancelChanges: function () {
            window.location.reload();
            this.ConcurrencyAlert.hide();
        },

        //For Concurrency Saving
        overridePreviousChanges: function () {
            this.set("overrideChanges", true);
            this.updateVariant();
            this.ConcurrencyAlert.hide();
        },

        createVariant: function () {
            this.hideHeaderActionsMessage();
            var invalidFieldNames = [];
            if (!this.validateRequiredFields(invalidFieldNames)) {
                this.updateSaveButton(false);
                this.updateTabsIndicators();
                this.showRequiredFieldsMessages(invalidFieldNames);
                this.InProgress.hide();
                return;
            }

            this.displayInProgressMessage();
            var baseFields = this.VariantBaseFields.get("editFunctionBody");

            var currentLanguage = this.get("currentLanguage");

            var dataObject = {};
            this.addRendererFieldsToObject(dataObject, baseFields);

            var extendedFields = this.ExtendedFieldsFormRenderer.get("editFunctionBody");
            this.addRendererFieldsToObject(dataObject, extendedFields);

            dataObject.variant_Images = this.CommerceImages.get("images");
            dataObject.currentLanguage = currentLanguage;
            dataObject.templateId = this.get("templateId");
            dataObject.parentId = this.get("parentId");
            dataObject.name = this.Name.get("text") !== null ? this.Name.get("text").trim() : "";

            // Custom validation of Name
            if (!this.isNameValid(dataObject.name)) {
                return;
            }

            var self = this;

            $.ajax({
                url: "/api/sitecore/CommerceVariant/Create",
                type: "POST",
                headers: this.ajaxToken,
                data: dataObject,
                context: this,
                error: CommerceUtilities.IsAuthenticated,
                success: function (data) {
                    this.HeaderActionsMessageBar.removeMessages();
                    if (data.Status == "failed") {
                        this.hideInProgressMessage();
                        for (var i = 0; i < data.Errors.length; i++) {
                            this.HeaderActionsMessageBar.addMessage("error", data.Errors[i].ErrorMessage);
                            this.HeaderActionsMessageBar.set("isVisible", true);
                            $("[data-sc-id='" + data.Errors[i].ControlId + "']").addClass("sc-validation");
                        }                        
                        $(".sc-validation").change(function () {
                            self.updateValidationError($(this));
                        });
                        $(".sc-validation.sc-datepicker").click(function () {
                            self.updateValidationError($(this));
                        });
                    } else {
                        this.hideInProgressMessageCreate(true, "Variant Created");
                        this.setPageIsDirty(false);
                        this.set("newTargetId", data);
                        setTimeout($.proxy(this.redirectToSavedVariant, this), 10000);
                    }
                }
            });
        },

        redirectToSavedVariant: function () {
            window.location.assign('/sitecore/client/Applications/MerchandisingManager/VariantDetail?target=' + this.get("newTargetId"));
        },

        selectedTabChanged: function () {
            this.set("selectedTab", this.VariantTab.get("selectedTab"));
            var selectedTabId = this.get("selectedTab");

            if (selectedTabId == "{917CC948-91C6-40B0-9813-FF3AA89C1DA2}") { // Details tab
                this.ProductInventoryBaseFields.set("isVisible", false);
                this.VariantActionControl.set("isVisible", true);
                this.InventoryActionControl.set("isVisible", false);
                this.InventorExtendedFieldsExpander.set("isVisible", false);
            } else if (selectedTabId == "{CD3FF5FA-EA98-4965-91EE-2B7A400D0789}") { // Inventory
                this.VariantActionControl.set("isVisible", false);
                this.InventoryActionControl.set("isVisible", true);
                this.ProductInventoryBaseFields.set("isVisible", true);
                this.InventorExtendedFieldsExpander.set("isVisible", true);
            }

        },

        updateValidationError: function (obj) {
            obj.removeClass("sc-validation");
            this.updateTabsIndicators();
            if ($(".sc-validation").length === 0) {
                this.updateSaveButton(true);
                this.hideHeaderActionsMessage();
            }
        },

        updateSaveButton: function (status) {
            this.VariantActionControl.getAction("Save").isEnabled(status);
            if (this.InventoryActionControl) {
                this.InventoryActionControl.getAction("Save").isEnabled(status);
            }
        },

        updateTabsIndicators: function () {
            var tabDetailsStatusElement = $("li[data-tab-id=\\{917CC948-91C6-40B0-9813-FF3AA89C1DA2\\}] > a > label", this.tabStrip);
            var tabInventoryStatusElement = $("li[data-tab-id=\\{CD3FF5FA-EA98-4965-91EE-2B7A400D0789\\}] > a > label", this.tabStrip);
            var detailsCount = 0;
            var inventoryCount = 0;
            $(".sc-validation").each(function () {
                if ($(this).parent().hasClass("details")) {
                    detailsCount++;
                } else if ($(this).parent().hasClass("inventory")) {
                    inventoryCount++;
                }
            });

            if (detailsCount > 0) {
                tabDetailsStatusElement.html("<div class='warning16'>");
            } else {
                tabDetailsStatusElement.html("");
            }

            if (inventoryCount > 0) {
                tabInventoryStatusElement.html("<div class='warning16'>");
            } else {
                tabInventoryStatusElement.html("");
            }
        },

        loadInventoryDetails: function (variantId) {
            // Get the inventory details for this product
            if (this.hasInventory === true) {

                // Setup the Status combo box
                var statusValues = [];
                statusValues.push({ statusValue: "Enabled", statusName: "Enabled" });
                statusValues.push({ statusValue: "Disabled", statusName: "Disabled" });
                statusValues.push({ statusValue: "Ignored", statusName: "Ignored" });
                this.Status.set("items", statusValues);

                var self = this;
                
                $.ajax({
                    url: "/api/sitecore/CommerceVariant/GetInventoryDetailsForVariant",
                    type: "POST",
                    headers: this.ajaxToken,
                    data: {
                        id: variantId
                    },
                    context: this,
                    error: CommerceUtilities.IsAuthenticated,
                    success: function (data) {
                        self.setInventoryBaseFields(data, self);
                    }
                });
            }
        },

        setInventoryBaseFields: function (data, page) {
            if (data) {
                page.InventoryCatalogName.set("text", data.InventoryCatalogName);
                page.Backorderable.set("isChecked", data.Backorderable);
                page.Preorderable.set("isChecked", data.Preorderable);
                page.OnHandQuantity.set("text", data.OnhandQuantity);
                page.ExcessOnHandQuantity.set("text", data.ExcessOnhandQuantity);
                page.ReorderPoint.set("text", data.ReorderPoint);
                page.TargetQuantity.set("text", data.TargetQuantity);
                page.BackorderLimit.set("text", data.BackorderLimit);
                page.PreorderLimit.set("text", data.PreorderLimit);
                page.StockOutThreshold.set("text", data.StockOutThreshold);
                page.PreorderAvailabilityDate.set("value", data.PreorderAvailability);
                page.BackorderAvailabilityDate.set("value", data.BackorderAvailability);
                page.LastRestocked.set("value", data.LastRestocked);
                page.UnitOfMeasure.set("text", data.UnitOfMeasure);
                page.Memo.set("text", data.Memo);
                page.SkuLastModified.set("text", CommerceUtilities.formatInventoryDateString(data.LastModified));
                page.BackorderedQuantity.set("text", data.BackorderedQuantity);
                page.PreorderedQuantity.set("text", data.PreorderedQuantity);
                page.Status.set("selectedValue", data.Status);
            }
        },

        determineLanguageToSet: function () {

            var language = CommerceUtilities.loadPageVar("lang");

            if (!language) {
                language = cbp.prototype.getCookie("commerceLang");
            }

            if (language) {

                var cultures = this.Language.get("items");

                if (cultures === null || cultures.length === 0) {
                    return;
                }

                var cultureFound = false;
                var catalogLanguage = "";

                for (i = 0; i < cultures.length; i++) {
                    if (cultures[i].language == language) {
                        cultureFound = true;
                        break;
                    }
                }

                if (!cultureFound) {
                    for (i = 0; i < cultures.length; i++) {
                        var languagePart = cultures[i].language.substring(0, 2);
                        if (languagePart == language) {
                            catalogLanguage = cultures[i].language;
                            break;
                        }
                    }
                }

                if (cultureFound) {
                    this.set("currentLanguage", language);
                } else {
                    if (catalogLanguage) {
                        this.set("currentLanguage", catalogLanguage);
                    } else {
                        this.set("currentLanguage", "en-US");
                    }
                }
            }
            else {
                this.set("currentLanguage", "en-US");
            }
        }
    });

    return VariantDetail;
});